/*
 *  Copyright (C) 2018  Matwey V. Kornilov <matwey.kornilov@gmail.com>
 *                      Konstantin Malanchev <hombit@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <openglshaderprogram.h>

void OpenGLShaderProgram::setAttribPointerHelper(QOpenGLBuffer& buffer, const int index) {
	buffer.bind();

	setAttributeArray(index, (const GLfloat*)0, 2, 0);
	enableAttributeArray(index);
}

OpenGLShaderProgram::OpenGLShaderProgram(QObject *parent):
	QOpenGLShaderProgram(parent) {
}

OpenGLShaderProgram::~OpenGLShaderProgram() = default;

bool OpenGLShaderProgram::addFragmentShaderFromSourceCode(const QString& source) {
	return addShaderFromSourceCode(QOpenGLShader::Fragment, source);
}

bool OpenGLShaderProgram::addVertexShaderFromSourceCode(const QString& source) {
	return addShaderFromSourceCode(QOpenGLShader::Vertex, source);
}

void OpenGLShaderProgram::setMVPUniform(const QMatrix4x4& mvp) {
	setUniformValue("MVP", mvp);
}

void OpenGLShaderProgram::setCUniform(const std::array<GLfloat, 4>& array, const quint8& channels) {
	setUniformValueArray("c", array.data(), 1, channels);
}

void OpenGLShaderProgram::setZUniform(const std::array<GLfloat, 4>& array, const quint8& channels) {
	setUniformValueArray("z", array.data(), 1, channels);
}

void OpenGLShaderProgram::setLayerUniform(float layer) {
	setUniformValue("layer", layer);
}

void OpenGLShaderProgram::setVertexCoordAttribPointer(QOpenGLBuffer& buffer) {
	setAttribPointerHelper(buffer, vertex_coord_index);
}

void OpenGLShaderProgram::setVertexUVAttribPointer(QOpenGLBuffer& buffer) {
	setAttribPointerHelper(buffer, vertex_UV_index);
}

bool OpenGLShaderProgram::link() {
	bindAttributeLocation(vertex_coord_name, vertex_coord_index);
	bindAttributeLocation(vertex_UV_name, vertex_UV_index);

	if (!QOpenGLShaderProgram::link()) return false;
	if (!bind()) return false;

	setUniformValue("texture", image_texture_index);
	setUniformValue("colormap", colormap_texture_index);

	return true;
}

constexpr const char OpenGLShaderProgram::vertex_coord_name[];
constexpr const char OpenGLShaderProgram::vertex_UV_name[];
constexpr const int OpenGLShaderProgram::vertex_coord_index;
constexpr const int OpenGLShaderProgram::vertex_UV_index;
constexpr const int OpenGLShaderProgram::image_texture_index;
constexpr const int OpenGLShaderProgram::colormap_texture_index;
